#!/usr/bin/python3
# coding: utf-8

"""
Plot graphs with data given by operations

This script receives a file path as an argument and outputs, in the same folder
as where the input is located, a bunch of plots

- number of requested nodes
- submit time of jobs relative to the first one
- histogram with the number of processors
- pie chart with the proportion of jobs per state

DATA FROM OPERATIONS HAS TO BE PREPROCESSED TO VALID .CSV

BSC-CNS - Earth Sciences
Manuel Giménez de Castro Marciani
2022
"""

import pandas as pd
import datetime
import matplotlib.pyplot as plt
import sys


def number_of_nodes(trace: pd.DataFrame, file_name: str, dir_path: str) -> None:
    """
    Histogram of the number of nodes asked for each job
    :param trace: Input pandas Data Frame with data from REAL slurm data
    :param file_name: Original data file name
    :param dir_path: Directory where the plot will be saved
    :return: None
    """
    print(f"Plot number of requested nodes histogram...")
    plt.hist(trace["NNodes"], bins=100)
    plt.title(f"Distribution of Nodes requested\n obs_trace from {file_name}")
    plt.ylabel("Number of Nodes")
    plt.xlabel("Number of Jobs")
    plt.savefig(f"{dir_path}/{file_name}_nodes.png")
    plt.clf()


def processors(trace: pd.DataFrame, file_name: str, dir_path: str) -> None:
    """
    Histogram of the number of processors asked for each job
    :param trace: Input pandas Data Frame with data from REAL slurm data
    :param file_name: Original data file name
    :param dir_path: Directory where the plot will be saved
    :return: None
    """
    print(f"Plot number of requested cpus histogram...")
    plt.hist(trace["NCPUS"], bins=100)
    plt.title(f"Distribution of CPUs requested\n Data from {file_name}")
    plt.ylabel("Number of CPUs")
    plt.xlabel("Number of Jobs")
    plt.savefig(f"{dir_path}/{file_name}_cpus.png")
    plt.clf()


def submit_time(trace: pd.DataFrame, file_name: str, dir_path: str) -> None:
    """
    Histogram of the start time with respect to the first started job
    :param trace: Input pandas Data Frame with data from REAL slurm data
    :param file_name: Original data file name
    :param dir_path: Directory where the plot will be saved
    :return: None
    """
    print(f"Plot submit time histogram...")
    plt.hist(trace["Zeroed submit"])
    plt.title(f"Distribution of submission time requested\n wrt to the first submission. Data from {file_name}")
    plt.ylabel("Number of Jobs")
    plt.xlabel("Submit time in seconds wrt to the first submission")
    plt.savefig(f"{dir_path}/{file_name}_submit_time.png")
    plt.clf()


def state_jobs(trace: pd.DataFrame, file_name: str, dir_path: str) -> None:
    """
    Pie chart of the states of the jobs
    :param trace: Input pandas Data Frame with data from REAL slurm data
    :param file_name: Original data file name
    :param dir_path: Directory where the plot will be saved
    :return: None
    """
    print(f"Plot pie-chart with job status...")
    states = trace["State"].unique()
    num_per_state = []
    for state in states:
        num_per_state.append(len(trace[trace["State"] == state]))
    plt.pie(num_per_state, labels=states)
    plt.title(f"Number of jobs per state\nData from {file_name}")
    plt.savefig(f"{dir_path}/{file_name}_states.png")
    plt.clf()


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("You have to provide the path to the CSV with COMMA AS DELIMITER")
        sys.exit(1)

    file_path = sys.argv[1]
    data_name = file_path.split("/")[-1].split(".")[0]  # picks the file name without path and file extension
    data_path = file_path.rsplit("/", 1)[0]  # remove the file name from directory path
    obs_trace = pd.read_csv(file_path, sep=",")

    print(f"Preprocessing {data_name}")

    # Transform submit data to datetime to make it easier to compute date differences
    formatted_date = []
    for i in range(len(obs_trace["Submit"])):
        formatted_date.append(datetime.datetime.strptime(obs_trace["Submit"][i], "%Y-%m-%dT%H:%M:%S"))
    # 8 is the position of the column, just for aesthetics 
    obs_trace.insert(8, "Formatted submit", formatted_date, allow_duplicates=True)

    print(f"Add formatted submit column")

    # this array will be the submitted time wrt reference in the first submitted job
    reference = min(formatted_date)  # pick the first job as the reference
    zeroed_date = []
    # We take as the reference the submission of the first job
    for i in range(len(obs_trace["Formatted submit"])):
        # And take the time in seconds wrt to the reference
        zeroed_date.append((obs_trace["Formatted submit"][i]-reference).seconds)
    obs_trace.insert(9, "Zeroed submit", zeroed_date, allow_duplicates=True)

    print(f"Add zeroed submit column")

    # Transform elapsed format to seconds
    elapsed_in_seconds = []
    for i in range(len(obs_trace["Elapsed"])):
        current_data = obs_trace["Elapsed"][i].split(":")
        elapsed_in_seconds.append((int(current_data[0])*60 + int(current_data[1]))*60 + int(current_data[2]))
    obs_trace.insert(7, "Elapsed in seconds", elapsed_in_seconds, allow_duplicates=True)

    number_of_nodes(obs_trace, data_name, data_path)
    submit_time(obs_trace, data_name, data_path)
    processors(obs_trace, data_name, data_path)
    state_jobs(obs_trace, data_name, data_path)
